/***********************************************************************
Mental Maps module include file.
***********************************************************************/

/* Help text files.  These should be found in directory InfoPath */
#define SLVALOFFY 20
#define INTRO_HLP    "intro.hlp"
#define QUIZDONE_HLP "quizdone.hlp"
#define QUESTION_HLP "quest.hlp"
#define VERSION_INFO "version.txt"

#define CTYNAMEFIELDWIDTH 16 /* width of city name field in .cty files */
#define PCODEFIELDWIDTH 10   /* width of postal code field in .cty files */
#define MAPSCALE     8.37186  /* Scaling for the coords in map databases */
#define QUEST_NUM    4 /* Number of menu items for picking questions */

/* Used for setting up countrylist */
#define FILEPREFIXLEN   8
typedef char FilePrefixString[FILEPREFIXLEN+1];
#define CITYSUFFIX   ".cty"
#define DUMPCITYSUFFIX   ".dcy"
#define BDRSUFFIX    ".bdr"

/* Positions on menu for questions */
#define TEMPERATURE  0 /* Ask about temperature */
#define POPULATION   1 /* Ask about population */
#define COST         2 /* Ask about cost of living */
#define PREF         3 /* Ask about preference for city */

#define LOCATION     4 /* Also used by askcity */
#define TEMP2        5 /* Status for low temperature */
#define RELOCATION   6 /* Also used by askcity */
#define MAXASK      68 /* Maximum number of cities to ask about */

/* Store the student's answers to the various questions. */
typedef struct {
  long citynum;  /* Index in the city record list */
  long hi;
  long low;
  long pop;
  long cost;
  long pref;
} Answerrec;


typedef struct {
  boolean mark; /* TRUE if this city already used in quiz */
  int x;        /* Coordinates for city */
  int y;
  int hi;       /* High temperature */
  int low;      /* Low temperature */
  long pop;     /* City population */
  long cost;    /* Cost of living */
  BigString name;  /* City name (including state or country) */
  int labx;     /* Coordinates for city label */
  int laby;
} Cityrec;

typedef struct {
  FilePrefixString name;
  FilePrefixString warp;
  int num;  /* Number of city records for this country */
  long minhi, maxhi, minlow, maxlow, minpop, maxpop, mincost, maxcost;
  Cityrec *clist;
  int fudgex;
  int fudgey;
  boolean labx_avail;	/* TRUE iff labx's in dbase file */
  boolean laby_avail;	/* TRUE iff laby's in dbase file */
  boolean pop_avail;	/* TRUE iff population figures in dbase file */
  boolean low_avail;	/* TRUE iff low temp figures in dbase file */
  boolean hi_avail;	/* TRUE iff high temp figures in dbase file */
  boolean cost_avail;	/* TRUE iff cost of living figures in dbase file */
  boolean st_avail;	/* TRUE iff states in dbase file */
} Countryrec;


typedef struct {
  int x, y, wd, ht;  /* Define the box for the slider bar */
  GSColor barcolor, tickcolor; /* Color of the body and the tickmark, resp. */
  boolean writeval;  /* Write the slider value while dragging iff TRUE */
  long min, range;   /* Minimum value and total range of value (max-min) */
} Slider;

/* rectangle around a city label */
typedef struct {
  double x;          /* x coord. of lower left corner */
  double y;          /* y coord. of lower left corner */
  double wd;         /* width of label rectangle */
  double ht;         /* height of label rectangle */
} MyRect;

typedef struct {
  GSColor xy;
  GSColor xyminus1;
  GSColor xplus1y;
  GSColor xyplus1;
  GSColor xminus1y;
  GSColor xyminus2;
  GSColor xplus1yminus1;
  GSColor xplus2y;
  GSColor xplus1yplus1;
  GSColor xyplus2;
  GSColor xminus1yplus1;
  GSColor xminus2y;
  GSColor xminus1yminus1;
} City_circle_pixels;

typedef struct {
  boolean enabled;
  boolean checked;
} toggle_item_status_t;

/* mmdecl.c */
extern BigString CityFile;
extern BigString Datapath;
extern BigString Helpfile;
extern BigString Home;
extern BigString Infopath;
extern BigString InterfaceFile;
extern BigString MapFile;
extern BigString Outputlogmode;
extern BigString Units;
extern BigString UserName;
extern BigString VersionFile;
extern BigString WarpMapFile;
extern BigString DumpFileName;

extern GraphicsFileType GraphFileFmt;
extern int CurrFormatItem;

extern boolean Question_item[QUEST_NUM];
extern boolean DumpSuccess;
extern boolean DumpResultsActive;
extern boolean DumpWinActive;
extern boolean FinalMapActive;
extern boolean GetName;
extern boolean HelpWinActive;
extern boolean LoginInProcess;
extern boolean Move_labels;
extern boolean Moving;
extern boolean NotInStepThru;
extern boolean Questesc;
extern boolean QuestionWinActive;
extern boolean QuitWinActive;
extern boolean QuizInProgress;
extern boolean ReadDebugInput;
extern boolean ResultWinActive;
extern boolean SummaryTableWinActive;
extern boolean Show_results;
extern boolean StepThru;
extern boolean Use_bstore;

extern char *StatePCodeTable[][2];

extern int AlertNumber;
extern int Citycnt;
extern int Curr_city;
extern int Curr_country;
extern int Default_country;
extern int Levelsback;
extern int Move_index;
extern int MovedLabel;
extern int MovedCity;
extern int Num_cities;
extern int Num_countries;
extern int Prev_quest_status;
extern int Quest_status;
extern int ResultWinCity;
extern int Savemsg;
extern int StepCount;

extern long *Slideval;

extern Countryrec *City;

extern Answerrec Answers[MAXASK];

extern Slider Currslide;

extern City_circle_pixels Citycirclepixels[68];

/* mmmisc.c */
void	askcity                  (void);
void    askquest                 (int, char*, boolean, long, long, char*, long);
boolean char_is_legal            (char);
void    check_off                (int);
boolean clearforlabel            (MyRect label);
boolean do_moving                (void);
void    draw_city_circle         (int, int, GSColor, City_circle_pixels*);
void    draw_map                 (char*, boolean);
void    draw_warped_map          (void);
void    dump_db_file             (void);
void    erase_city_circle        (int, int, City_circle_pixels);
void    finish_quiz              (void);
char   *fullname                 (char*);
boolean helpdone                 (void);
void    log_answers              (void);
boolean login_return_func        (void);
void    prompt_for_location      (void);
boolean rects_overlap            (MyRect, MyRect);
boolean redraw_all               (void);
void    show_city                (int city);
void	show_final_map           (void);
void	show_results             (int);
void	show_summary_table       (void);
void	step_thru                (void);

/*--- iroutine.c ---*/
void    highlightfield           (BigString, BigString, GSColor, GSColor,
                                  GSColor);

/* slider.c */
double  drawslider(int, int, int, int, GSColor, GSColor, int);
